"""
    domonic.i18n.da
    ====================================

    html tags in Danish

    first translated to Danish using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

krop = body
hovedet = head
# script = script
stil = style
# portal = portal
stærk = strong
blokcitat = blockquote
tabel = table
# form = form
etiket = label
indsend = submit
titel = title
# noscript = noscript
afsnit = section
# nav = nav
artikel = article
tilside = aside
hgruppe = hgroup
adresse = address
figur = figure
figcaption = figcaption
lille = small
citere = cite
kode = code
# sub = sub
# sup = sup
mærke = mark
rubin = ruby
# span = span
# iframe = iframe
# video = video
lyd = audio
lærred = canvas
billedtekst = caption
colgroup = colgroup
krop = tbody
# thead = thead
tfod = tfoot
feltsæt = fieldset
legende = legend
knap = button
vælg = select
dataliste = datalist
# optgroup = optgroup
mulighed = option
tekstområde = textarea
# output = output
fremskridt = progress
# meter = meter
detaljer = details
resumé = summary
# menu = menu
# menuitem = menuitem
skrifttype = font
overskrift = header
sidefod = footer
# base = base
# link = link
# meta = meta
# img = img
indlejre = embed
kilde = source
spor = track
område = area
# col = col
# input = input
kommando = command
hoved = main
# basefont = basefont
centrum = center
indlejre = embed
skrifttype = font
isindeks = isindex
liste = listing
klartekst = plaintext
strejke = strike
skabelon = template
billede = picture
# doctype = doctype
kommentar = comment
