"""
    domonic.i18n.sw
    ====================================

    html tags in Swahili

    translated to Swahili using google translate.
    send a pull request if you have a better translation.

    html(
        kichwa(),
        mwili(
            div('Salamu, Dunia')
        )
    )
"""
from domonic.html import *

# tags
mwili = body
kichwa = head
hati = script
mtindo = style
# portal = portal
nguvu = strong
# blockquote = blockquote
meza = table
fomu = form
lebo = label
kuwasilisha = submit
# kichwa = title
# noscript = noscript
sehemu = section
# nav = nav
makala = article
pembeni = aside
kundi = hgroup
anwani = address
takwimu = figure
# figcaption = figcaption
ndogo = small
taja = cite
kanuni = code
ndogo = sub
chakula = sup
alama = mark
# ruby = ruby
muda = span
# iframe = iframe
# video = video
sauti = audio
turubai = canvas
maelezo = caption
kikundi = colgroup
# mwili = tbody
# kichwa = thead
mguu = tfoot
uwanja = fieldset
hadithi = legend
kitufe = button
chagua = select
orodhayadata = datalist
optgroup = optgroup
chaguo = option
eneolamaandishi = textarea
pato = output
maendeleo = progress
mita = meter
maelezo = details
muhtasari = summary
menyu = menu
# menyu = menuitem
fonti = font
# kichwa = header
chini = footer
msingi = base
kiungo = link
# meta = meta
# img = img
# embed = embed
chanzo = source
wimbo = track
eneo = area
col = col
pembejeo = input
amri = command
kuu = main
fonti = basefont
kituo = center
# embed = embed
fonti = font
# isindex = isindex
orodha = listing
maandishiwazi = plaintext
mgomo = strike
# template = template
picha = picture
# doctype = doctype
maoni = comment
