"""
    domonic.i18n.mi
    ====================================

    html tags in Maori

    first translated to Maori using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

tinana = body
upoko = head
tuhi = script
kāhua = style
tomokanga = portal
kaha = strong
kupuporaka = blockquote
tepu = table
ahua = form
tapanga = label
tuku = submit
taitara = title
# noscript = noscript
wahanga = section
# nav = nav
tuhinga = article
tahae = aside
# hgroup = hgroup
wāhitau = address
ahua = figure
whakakitenga = figcaption
iti = small
whakahua = cite
waehere = code
iti = sub
sup = sup
tohu = mark
rupi = ruby
whanganga = span
iframe = iframe
ataata = video
ororongo = audio
kākahu = canvas
tapanga = caption
roopuroopu = colgroup
tinana = tbody
# upoko = thead
waewae = tfoot
huingamara = fieldset
pūrākau = legend
patene = button
whiriwhiri = select
rārangiraraunga = datalist
optgroup = optgroup
kōwhiringa = option
textarea = textarea
putanga = output
ahuwhakamua = progress
mita = meter
taipitopito = details
whakarāpopototanga = summary
tahua = menu
tahua = menuitem
momotuhi = font
pane = header
hiku = footer
turanga = base
hono = link
meta = meta
img = img
tāmau = embed
puna = source
ara = track
rohe = area
# col = col
tāuru = input
whakahau = command
matua = main
momotuhituranga = basefont
pokapū = center
tāmau = embed
momotuhi = font
isindex = isindex
whakarārangi = listing
Kuputuhi = plaintext
patuki = strike
tauira = template
pikitia = picture
momokorero = doctype
korero = comment
