"""
    domonic.webapi.mediacapabilities
    ====================================
    https://developer.mozilla.org/en-US/docs/Web/API/Media_Capabilities_API
"""

class MediaCapabilities:
    
    @staticmethod
    def encodingInfo(mediaEncodingConfiguration):
        pass

    @staticmethod
    def decodingInfo(mediaDecodingConfiguration):
        pass


# MediaDecodingConfiguration
# MediaEncodingConfiguration
# VideoConfiguration
# AudioConfiguration
