"""
    domonic.i18n.ch
    ====================================

    html tags in Chinese

    first translated to Chinese using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

Shēntǐ = 身体 = body
tóu = 头 = head
jiǎoběn = 脚本 = script
fēnggé = 风格 = style
chuánsòngmén = 传送门 = portal
qiáng = 强 = strong
kuàiyǐnyòng = 块引用 = blockquote
biǎo = 表 = table
xíngshì = 形式 = form
biāoqiān = 标签 = label
tíjiāo = 提交 = submit
biāotí = 标题 = title
# noscript = noscript = noscript
bùfèn = 部分 = section
dǎoháng = 导航 = nav
wénzhāng = 文章 = article
yībiān = 一边 = aside
# hgroup = hgroup = hgroup
dìzhǐ = 地址 = address
shùzì = 数字 = figure
# figcaption = figcaption = figcaption
xiǎo = 小 = small
yǐnyòng = 引用 = cite
dàimǎ = 代码 = code
zi = 子 = sub
chāo = 超 = sup
biāojì = 标记 = mark
hóngbǎoshí = 红宝石 = ruby
kuàdù = 跨度 = span
# iframe = iframe = iframe
shìpín = 视频 = video
yīnpín = 音频 = audio
huàbù = 画布 = canvas
biāotí = 标题 = caption
zǔqún = 组群 = colgroup
shēntǐ = 身体 = tbody
# tóu = 头 = thead
jiǎo = 脚 = tfoot
zìduànjí = 字段集 = fieldset
chuánshuō = 传说 = legend
ànniǔ = 按钮 = button
xuǎnzé = 选择 = select
shùjùlièbiǎo = 数据列表 = datalist
xuǎnzézǔ = 选择组 = optgroup
xuǎnxiàng = 选项 = option
wénběnqūyù = 文本区域 = textarea
shūchū = 输出 = output
jìnbù = 进步 = progress
mǐ = 米 = meter
xiángqíng = 详情 = details
zǒngjié = 总结 = summary
càidān = 菜单 = menu
càidānxiàng = 菜单项 = menuitem
zìtǐ = 字体 = font
biāotí = 标题 = header
yèjiǎo = 页脚 = footer
jīshù = 基数 = base
liànjiē = 链接 = link
yuán = 元 = meta
túpiàn = 图片 = img
qiànrù = 嵌入 = embed
láiyuán = 来源 = source
guǐdào = 轨道 = track
miànjī = 面积 = area
shàngxiào = 上校 = col
shūrù = 输入 = input
mìnglìng = 命令 = command
zhǔyào = 主要 = main
jīběnzìtǐ = 基本字体 = basefont
zhōngxīn = 中心 = center
qiànrù = 嵌入 = embed
zìtǐ = 字体 = font
# isindex = isindex = isindex
lièbiǎo = 列表 = listing
míngwén = 明文 = plaintext
bàgōng = 罢工 = strike
múbǎn = 模板 = template
túpiàn = 图片 = picture
wéndànglèixíng = 文档类型 = doctype
pínglùn = 评论 = comment
