"""
    domonic.i18n.es
    ====================================

    html tags in Spanish

    translated to Spanish using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

cuerpo = body
cabeza = head
# script = script
estilo = style
# portal = portal
fuerte = strong
# blockquote = blockquote
mesa = table
formulario = form
etiqueta = label
enviar = submit
# title = title
# noscript = noscript
sección = section
# nav = nav
articulo = article
aparte = aside
hgroup = hgroup
dirección = address
figura = figure
# figcaption = figcaption
pequeño = small
cite = cite
código = code
# sub = sub
# sup = sup
marca = mark
ruby = ruby
# span = span
# iframe = iframe
# video = video
# audio = audio
lienzo = canvas
subtítulo = caption
colgroup = colgroup
# tbody = tbody
# thead = thead
# tfoot = tfoot
fieldset = fieldset
leyenda = legend
botón = button
seleccionar = select
# datalist = datalist
# optgroup = optgroup
opción = option
# textarea = textarea
salida = output
progreso = progress
metro = meter
detalles = details
resumen = summary
# menu = menu
# menuitem = menuitem
fuente = font
encabezado = header
piedepágina = footer
base = base
enlace = link
meta = meta
# img = img
# embed = embed
fuente = source
# track = track
# area = area
# col = col
entrada = input
comando = command
principal = main
# basefont = basefont
centro = center
fuente = font
# isindex = isindex
listado = listing
textosinformato = plaintext
# strike = strike
plantilla = template
imagen = picture
# doctype = doctype
comentario = comment